package com.yiqi.system.service.impl;

import java.util.List;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yiqi.common.utils.DateUtils;
import com.yiqi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.yiqi.system.mapper.SysMessageConfigMapper;
import com.yiqi.system.domain.SysMessageConfig;
import com.yiqi.system.service.ISysMessageConfigService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * 消息配置Service业务层处理
 *
 * @author douya
 * @date 2024-11-22
 */
@Service
public class SysMessageConfigServiceImpl extends ServiceImpl<SysMessageConfigMapper, SysMessageConfig> implements ISysMessageConfigService {

    //<editor-folder desc="基础韩素">

    /**
     * 查询消息配置
     *
     * @param id 消息配置主键
     * @return 消息配置
     */
    @Override
    public SysMessageConfig selectSysMessageConfigById(Long id) {
        return baseMapper.selectSysMessageConfigById(id);
    }

    /**
     * 查询消息配置列表
     *
     * @param sysMessageConfig 消息配置
     * @return 消息配置
     */
    @Override
    public List<SysMessageConfig> selectSysMessageConfigList(SysMessageConfig sysMessageConfig) {
        return baseMapper.selectSysMessageConfigList(sysMessageConfig);
    }

    /**
     * 新增消息配置
     *
     * @param sysMessageConfig 消息配置
     * @return 结果
     */
    @Override
    public int insertSysMessageConfig(SysMessageConfig sysMessageConfig) {
        sysMessageConfig.setCreateTime(DateUtils.getNowDate());
        return baseMapper.insertSysMessageConfig(sysMessageConfig);
    }

    /**
     * 修改消息配置
     *
     * @param sysMessageConfig 消息配置
     * @return 结果
     */
    @Override
    public int updateSysMessageConfig(SysMessageConfig sysMessageConfig) {
        sysMessageConfig.setUpdateTime(DateUtils.getNowDate());
        return baseMapper.updateSysMessageConfig(sysMessageConfig);
    }

    /**
     * 批量删除消息配置
     *
     * @param ids 需要删除的消息配置主键
     * @return 结果
     */
    @Override
    public int deleteSysMessageConfigByIds(Long[] ids) {
        return baseMapper.deleteSysMessageConfigByIds(ids);
    }

    /**
     * 删除消息配置信息
     *
     * @param id 消息配置主键
     * @return 结果
     */
    @Override
    public int deleteSysMessageConfigById(Long id) {
        return baseMapper.deleteSysMessageConfigById(id);
    }

    /**
     * 修改消息配置状态
     *
     * @param sysMessageConfig 消息配置
     * @return 结果
     */
    @Override
    public int updateSysMessageConfigStatus(SysMessageConfig sysMessageConfig) {
        sysMessageConfig.setUpdateById(SecurityUtils.getUserId());
        sysMessageConfig.setUpdateBy(SecurityUtils.getUsername());
        sysMessageConfig.setUpdateTime(DateUtils.getNowDate());
        return baseMapper.updateSysMessageConfig(sysMessageConfig);
    }

    @Override
    public SysMessageConfig selectSysMessageConfigByCode(String code) {
        QueryWrapper<SysMessageConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysMessageConfig::getCode, code);
        queryWrapper.lambda().eq(SysMessageConfig::getStatus, 1);
        return baseMapper.selectOne(queryWrapper);
    }
}
